// Design: code
// Description: Digital code key
// Author: Jorge Juan <jjchico@dte.us.es>
// Copyright Universidad de Sevilla, Spain
// Date: 14-12-2011

////////////////////////////////////////////////////////////////////////////////                                                                         //
// This file is free software: you can redistribute it and/or modify it under //
// the terms of the GNU General Public License as published by the Free       //
// Software Foundation, either version 3 of the License, or (at your option)  //
// any later version. You can access a copy of the GNU General Public License //
// at <http://www.gnu.org/licenses/>.                                         //
////////////////////////////////////////////////////////////////////////////////

/*
   This file describes a siple digital code key. The system has 4 inputs (b0,
   b1, b2 and b3) and one output (z). The output activates (1) when the inputs
   are "pressed" (1) in this sequence: b0, b2, b1.
   
   It is assummed that inputs are connected to unfiltered (bouncing) push
   buttons, so the state machine will neet to take into account the possible
   bouncing of the inputs.
   
   Key sequence can start at any time. When the output is active (door open)
   any button press will make it close (except b1, which is the last button
   of the key).
   
   The system has an asynchronous reset that sets the system to an initial
   known state.
*/

`timescale 1ns / 1ps

//////////////////////////////////////////////////////////////////////////
// Digital code key FSM                                                 //
//////////////////////////////////////////////////////////////////////////

 module code(
 	input wire clk,		// clock
 	input wire reset,	// reset
	input wire [3:0] b,	// input buttons
 	output reg z		// output
 	);

 	// State encoding
 	parameter [1:0]
 		A = 2'b00,	// waiting first digit
 		B = 2'b01,	// first digit (b0) ok
 		C = 2'b10,	// second digit (b2) ok
 		D = 2'b11;	// third digit (b1) ok. Activate output

	// State and next state variables
	reg [1:0] state, next_state;

	// State change process (sequential part)
	always @(posedge clk, posedge reset)
		if (reset)
			state <= A;
		else
			state <= next_state;

	// Next state calculation process (combinational)
	always @* begin
		next_state = 2'bxx;
		case (state)
		A:
			if (b == 4'b0001)
				next_state = B;
			else
				next_state = A;
		B:
			// if b0 is still pressed or released, stay in B
			// if b2 is pressed move to C
			// any other button is a wrong code so go to A
			if (b[3:1] == 3'b000)
				next_state = B;
			else if (b == 4'b0100)
				next_state = C;
			else
				next_state = A;
		C:
			/* your code here */
		D:
			/* your code here */
		endcase
	end

	// Output calculation process (combinational)
	always @* begin
		/* your code here */
	end
endmodule // code
